"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ApolloServer = void 0;
const cors_1 = __importDefault(require("@koa/cors"));
const koa_bodyparser_1 = __importDefault(require("koa-bodyparser"));
const koa_compose_1 = __importDefault(require("koa-compose"));
const graphql_playground_html_1 = require("@apollographql/graphql-playground-html");
const apollo_server_core_1 = require("apollo-server-core");
const accepts_1 = __importDefault(require("accepts"));
const middlewareFromPath = (path, middleware) => (ctx, next) => {
    if (ctx.path === path) {
        return middleware(ctx, next);
    }
    else {
        return next();
    }
};
class ApolloServer extends apollo_server_core_1.ApolloServerBase {
    async createGraphQLServerOptions(ctx) {
        return super.graphQLServerOptions({ ctx });
    }
    applyMiddleware({ app, ...rest }) {
        app.use(this.getMiddleware(rest));
    }
    getMiddleware({ path, cors, bodyParserConfig, disableHealthCheck, onHealthCheck, } = {}) {
        if (!path)
            path = '/graphql';
        this.ensureStarting();
        const middlewares = [];
        if (!disableHealthCheck) {
            middlewares.push(middlewareFromPath('/.well-known/apollo/server-health', async (ctx) => {
                ctx.set('Content-Type', 'application/health+json');
                if (onHealthCheck) {
                    try {
                        await onHealthCheck(ctx);
                        ctx.body = { status: 'pass' };
                    }
                    catch (e) {
                        ctx.status = 503;
                        ctx.body = { status: 'fail' };
                    }
                }
                else {
                    ctx.body = { status: 'pass' };
                }
            }));
        }
        this.graphqlPath = path;
        if (cors === true) {
            middlewares.push(middlewareFromPath(path, cors_1.default()));
        }
        else if (cors !== false) {
            middlewares.push(middlewareFromPath(path, cors_1.default(cors)));
        }
        if (bodyParserConfig === true) {
            middlewares.push(middlewareFromPath(path, koa_bodyparser_1.default()));
        }
        else if (bodyParserConfig !== false) {
            middlewares.push(middlewareFromPath(path, koa_bodyparser_1.default(bodyParserConfig)));
        }
        middlewares.push(middlewareFromPath(path, async (ctx) => {
            if (ctx.request.method === 'OPTIONS') {
                ctx.status = 204;
                ctx.body = '';
                return;
            }
            if (this.playgroundOptions && ctx.request.method === 'GET') {
                const accept = accepts_1.default(ctx.req);
                const types = accept.types();
                const prefersHTML = types.find((x) => x === 'text/html' || x === 'application/json') === 'text/html';
                if (prefersHTML) {
                    const playgroundRenderPageOptions = {
                        endpoint: path,
                        ...this.playgroundOptions,
                    };
                    ctx.set('Content-Type', 'text/html');
                    const playground = graphql_playground_html_1.renderPlaygroundPage(playgroundRenderPageOptions);
                    ctx.body = playground;
                    return;
                }
            }
            try {
                const { graphqlResponse, responseInit } = await apollo_server_core_1.runHttpQuery([ctx], {
                    method: ctx.request.method,
                    options: () => this.createGraphQLServerOptions(ctx),
                    query: ctx.request.method === 'POST'
                        ?
                            ctx.request.body || ctx.req.body
                        : ctx.request.query,
                    request: apollo_server_core_1.convertNodeHttpToRequest(ctx.req),
                });
                if (responseInit.headers) {
                    Object.entries(responseInit.headers).forEach(([headerName, value]) => ctx.set(headerName, value));
                }
                ctx.body = graphqlResponse;
            }
            catch (e) {
                const error = e;
                if ('HttpQueryError' !== error.name) {
                    throw error;
                }
                if (error.headers) {
                    Object.entries(error.headers).forEach(([headerName, value]) => ctx.set(headerName, value));
                }
                ctx.status = error.statusCode;
                ctx.body = error.message;
            }
        }));
        return koa_compose_1.default(middlewares);
    }
}
exports.ApolloServer = ApolloServer;
//# sourceMappingURL=ApolloServer.js.map